@@ -1,23 +1,23 @@
 /* JitterLisp: Jittery VM code generator.
 
-   Copyright (C) 2018, 2019, 2020 Luca Saiu
+   Copyright (C) 2018, 2019, 2020, 2021 Luca Saiu
    Written by Luca Saiu
 
    This file is part of the JitterLisp language implementation, distributed as
-   an example along with Jitter under the same license.
+   an example along with GNU Jitter under the same license.
 
-   Jitter is free software: you can redistribute it and/or modify
+   GNU Jitter is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
 
-   Jitter is distributed in the hope that it will be useful,
+   GNU Jitter is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with Jitter.  If not, see <http://www.gnu.org/licenses/>. */
+   along with GNU Jitter.  If not, see <https://www.gnu.org/licenses/>. */
 
 
 #include "jitterlisp-code-generator.h"
@@ -693,12 +693,10 @@ jitterlisp_compile (struct jitterlisp_closure *c,
   jitterlisp_generate_jittery (code_as_sexpression, & r, & er);
 
   /* Make sure there are enough slow registers in the VM state to run this code
-     as well.  Since we only have one state struct, this is easy: just update it
-     now, after we have generated a new executable routine and we know how many
-     registers this will take.  Of course if we already have more, none will be
-     removed. */
-  jitterlispvm_ensure_enough_slow_registers_for_executable_routine
-     (er, & jitterlispvm_state);
+     as well. */
+  jitter_uint slow_register_per_class_no = er->slow_register_per_class_no;
+  if (jitterlisp_slow_register_per_class_no < slow_register_per_class_no)
+    jitterlisp_slow_register_per_class_no = slow_register_per_class_no;
 
   /* Set closure fields. */
   if (jitterlisp_settings.free_routines)
