@@ -1,23 +1,23 @@
 /* Jitter: hash table data structure header.
 
    Copyright (C) 2017, 2018 Luca Saiu
-   Updated in 2021 by Luca Saiu
+   Updated in 2021 and 2022 by Luca Saiu
    Written by Luca Saiu
 
-   This file is part of Jitter.
+   This file is part of GNU Jitter.
 
-   Jitter is free software: you can redistribute it and/or modify
+   GNU Jitter is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
 
-   Jitter is distributed in the hope that it will be useful,
+   GNU Jitter is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with Jitter.  If not, see <http://www.gnu.org/licenses/>. */
+   along with GNU Jitter.  If not, see <https://www.gnu.org/licenses/>. */
 
 
 #ifndef JITTER_HASH_H_
@@ -196,18 +196,19 @@ jitter_hash_table_remove (struct jitter_hash_table *t,
 
 /* A function performing some work on a hash binding. */
 typedef
-void (* jitter_hash_for_all_bindings_function)
+void (* jitter_hash_for_all_bindings_function_f)
      (const union jitter_word key,
-      const union jitter_word value,
+      union jitter_word *value,
       void *extra_datum);
 
 /* Call the given function once on each binding of the pointed hash table, in
    some unspecified order.  The last argument is passed as is to the worker
    function.
-   The function must not change the hash table content. */
+   The function must not change the hash table keys and cannot add or remove
+   bindings, but is allowed to change data. */
 void
 jitter_hash_for_all_bindings (const struct jitter_hash_table *t,
-                              jitter_hash_for_all_bindings_function f,
+                              jitter_hash_for_all_bindings_function_f f,
                               void *extra_datum);
 
 
