@@ -146,7 +146,14 @@ static jitter_int
 jitter_distance_from (const char *destination_p,
                       const char *branch_instruction_p)
 {
-  return destination_p - branch_instruction_p;
+  return (/* To avoid undefined behaviour subtract two unsigned integers --
+             which may wrap around -- and then convert the result, which will be
+             on two's complement on this platform, to a signed value.  We are
+             still technically in undefined behaviour's territory because the
+             two pointers do not belong to the same buffer, but GCC can handle
+             this. */
+          (jitter_int)
+          ((jitter_uint) destination_p - (jitter_uint) from_p));
 }
 
 enum jitter_snippet_to_patch
