@@ -1,6 +1,6 @@
 /* ios-dev-mem.c - Memory IO devices.  */
 
-/* Copyright (C) 2020, 2021 Jose E. Marchesi */
+/* Copyright (C) 2020, 2021, 2022, 2023 Jose E. Marchesi */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,7 +24,7 @@
 #include "ios.h"
 #include "ios-dev.h"
 
-/* State asociated with a memory device.  */
+/* State associated with a memory device.  */
 
 struct ios_dev_mem
 {
@@ -35,43 +35,58 @@ struct ios_dev_mem
 
 #define MEM_STEP (512 * 8)
 
-static char *
+static const char *
 ios_dev_mem_get_if_name () {
   return "MEMORY";
 }
 
-static int
-ios_dev_mem_handler_normalize (const char *handler, uint64_t flags, char **newhandler)
+static char *
+ios_dev_mem_handler_normalize (const char *handler, uint64_t flags, int *error)
 {
+  char *new_handler = NULL;
   if (handler[0] == '*' && handler[strlen (handler) - 1] == '*')
     {
-      *newhandler = strdup (handler);
-      if (*newhandler == NULL)
-        return IOD_ENOMEM;
+      new_handler = strdup (handler);
+      if (new_handler == NULL && error)
+        *error = IOD_ENOMEM;
     }
-  return IOD_OK;
+  if (error)
+    *error = IOD_OK;
+  return new_handler;
 }
 
-static int
-ios_dev_mem_open (const char *handler, uint64_t flags, void **dev)
+static void *
+ios_dev_mem_open (const char *handler, uint64_t flags, int *error,
+                  void *data __attribute__ ((unused)))
 {
+  int internal_error = IOD_ERROR;
   struct ios_dev_mem *mio = malloc (sizeof (struct ios_dev_mem));
 
   if (!mio)
-    return IOD_ENOMEM;
+    {
+      internal_error = IOD_ENOMEM;
+      goto err;
+    }
 
   mio->pointer = calloc (MEM_STEP, 1);
   if (!mio->pointer)
     {
-      free (mio);
-      return IOD_ENOMEM;
+      internal_error = IOD_ENOMEM;
+      goto err;
     }
 
   mio->size = MEM_STEP;
   mio->flags = IOS_F_READ | IOS_F_WRITE;
 
-  *dev = mio;
-  return IOD_OK;
+  if (error)
+    *error = IOD_OK;
+  return mio;
+
+err:
+  free (mio);
+  if (error)
+    *error = internal_error;
+  return NULL;
 }
 
 static int
@@ -82,7 +97,7 @@ ios_dev_mem_close (void *iod)
   free (mio->pointer);
   free (mio);
 
-  return 1;
+  return IOD_OK;
 }
 
 static uint64_t
