@@ -722,7 +722,7 @@ ios_read_int_common (ios io, ios_off offset, int flags,
     return IOS_OK;
 
   default:
-    assert (0);
+    PK_UNREACHABLE ();
   }
 }
 
@@ -1190,7 +1190,7 @@ ios_write_int_fast (ios io, ios_off offset, int flags,
       break;
 
     default:
-      assert (0);
+      PK_UNREACHABLE ();
       break;
     }
 
@@ -1275,7 +1275,7 @@ ios_write_int_common (ios io, ios_off offset, int flags,
       /* Convert to the little endian format. For example a 12-bit-long
          number's bits get reordered as 7-6-5-4-3-2-1-0-11-10-9-8
          with leading 0s.  */
-      if (bits <= 16)
+      if (bits < 16)
         value = ((value & 0xff) << (bits % 8))
                 | (value & 0xff00) >> 8;
       else
@@ -1302,7 +1302,7 @@ ios_write_int_common (ios io, ios_off offset, int flags,
       /* Convert to the little endian format. For example a 12-bit-long
          number's bits get reordered as 7-6-5-4-3-2-1-0-11-10-9-8
          with leading 0s.  */
-      if (bits <= 24)
+      if (bits < 24)
         value = ((value & 0xff) << (8 + bits % 8))
                 | (value & 0xff00) >> (8 - bits % 8)
                 | (value & 0xff0000) >> 16;
@@ -1332,7 +1332,7 @@ ios_write_int_common (ios io, ios_off offset, int flags,
       /* Convert to the little endian format. For example a 12-bit-long
          number's bits get reordered as 7-6-5-4-3-2-1-0-11-10-9-8
          with leading 0s.  */
-      if (bits <= 32)
+      if (bits < 32)
         value = ((value & 0xff) << (16 + bits % 8))
                 | (value & 0xff00) << (bits % 8)
                 | (value & 0xff0000) >> (16 - bits % 8)
@@ -1365,7 +1365,7 @@ ios_write_int_common (ios io, ios_off offset, int flags,
       /* Convert to the little endian format. For example a 12-bit-long
          number's bits get reordered as 7-6-5-4-3-2-1-0-11-10-9-8
          with leading 0s.  */
-      if (bits <= 40)
+      if (bits < 40)
         value = ((value & 0xff) << (24 + bits % 8))
                 | (value & 0xff00) << (8 + bits % 8)
                 | (value & 0xff0000) >> (8 - bits % 8)
@@ -1401,7 +1401,7 @@ ios_write_int_common (ios io, ios_off offset, int flags,
       /* Convert to the little endian format. For example a 12-bit-long
          number's bits get reordered as 7-6-5-4-3-2-1-0-11-10-9-8
          with leading 0s.  */
-      if (bits <= 48)
+      if (bits < 48)
         value = ((value & 0xff) << (32 + bits % 8))
                 | (value & 0xff00) << (16 + bits % 8)
                 | (value & 0xff0000) << (bits % 8)
@@ -1440,7 +1440,7 @@ ios_write_int_common (ios io, ios_off offset, int flags,
       /* Convert to the little endian format. For example a 12-bit-long
          number's bits get reordered as 7-6-5-4-3-2-1-0-11-10-9-8
          with leading 0s.  */
-      if (bits <= 56)
+      if (bits < 56)
         value = ((value & 0xff) << (40 + bits % 8))
                 | (value & 0xff00) << (24 + bits % 8)
                 | (value & 0xff0000) << (8 + bits % 8)
@@ -1482,14 +1482,20 @@ ios_write_int_common (ios io, ios_off offset, int flags,
       /* Convert to the little endian format. For example a 12-bit-long
          number's bits get reordered as 7-6-5-4-3-2-1-0-11-10-9-8
          with leading 0s.  */
-      value = ((value & 0xff) << (48 + bits % 8))
-              | (value & 0xff00) << (32 + bits % 8)
-              | (value & 0xff0000) << (16 + bits % 8)
-              | (value & 0xff000000) << (bits % 8)
-              | (value & 0xff00000000) >> (16 - bits % 8)
-              | (value & 0xff0000000000) >> (32 - bits % 8)
-              | (value & 0xff000000000000) >> (48 - bits % 8)
-              | (value & 0xff00000000000000) >> 56;
+      if (bits < 64)
+        value = ((value & 0xff) << (48 + bits % 8))
+                | (value & 0xff00) << (32 + bits % 8)
+                | (value & 0xff0000) << (16 + bits % 8)
+                | (value & 0xff000000) << (bits % 8)
+                | (value & 0xff00000000) >> (16 - bits % 8)
+                | (value & 0xff0000000000) >> (32 - bits % 8)
+                | (value & 0xff000000000000) >> (48 - bits % 8)
+                | (value & 0xff00000000000000) >> 56;
+      else
+        {
+          assert (bits == 64);
+          value = bswap_64 (value);
+        }
     }
     c[0] |= value >> (56 + lastbyte_bits);
     c[1] = (value >> (48 + lastbyte_bits)) & 0xff;
@@ -1504,7 +1510,7 @@ ios_write_int_common (ios io, ios_off offset, int flags,
     return IOS_OK;
 
   default:
-    assert (0);
+    PK_UNREACHABLE ();
   }
 }
 
