@@ -1,6 +1,6 @@
 /* pk-val.c - poke values.  */
 
-/* Copyright (C) 2020, 2021 Jose E. Marchesi */
+/* Copyright (C) 2020, 2021, 2022, 2023 Jose E. Marchesi */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,19 +26,7 @@
 pk_val
 pk_make_int (int64_t value, int size)
 {
-  pk_val new;
-
-  /* At the moment poke integers are limited to a maximum number of
-     bits.  */
-  if (size > 64)
-    return PK_NULL;
-
-  if (size <= 32)
-    new = pvm_make_int (value, size);
-  else
-    new = pvm_make_long (value, size);
-
-  return new;
+  return pvm_make_signed_integral (value, size);
 }
 
 int64_t
@@ -62,19 +50,7 @@ pk_int_size (pk_val val)
 pk_val
 pk_make_uint (uint64_t value, int size)
 {
-  pk_val new;
-
-  /* At the moment poke integers are limited to a maximum number of
-     bits.  */
-  if (size > 64)
-    return PK_NULL;
-
-  if (size <= 32)
-    new = pvm_make_uint (value, size);
-  else
-    new = pvm_make_ulong (value, size);
-
-  return new;
+  return pvm_make_unsigned_integral (value, size);
 }
 
 uint64_t
@@ -234,26 +210,64 @@ pk_type_code (pk_val val)
     {
     case PVM_TYPE_INTEGRAL:
       if ((pk_int_value (pk_integral_type_signed_p (val))))
-        return PK_INT;
+        return PK_TYPE_INT;
       else
-        return PK_UINT;
+        return PK_TYPE_UINT;
     case PVM_TYPE_STRING:
-      return PK_STRING;
+      return PK_TYPE_STRING;
     case PVM_TYPE_ARRAY:
-      return PK_ARRAY;
+      return PK_TYPE_ARRAY;
     case PVM_TYPE_STRUCT:
-      return PK_STRUCT;
+      return PK_TYPE_STRUCT;
     case PVM_TYPE_OFFSET:
-      return PK_OFFSET;
+      return PK_TYPE_OFFSET;
     case PVM_TYPE_CLOSURE:
-      return PK_CLOSURE;
-    case PVM_TYPE_ANY:
-      return PK_ANY;
+      return PK_TYPE_CLOSURE;
+    case PVM_TYPE_VOID:
+      return PK_TYPE_VOID;
     default:
-      return PK_UNKNOWN;
+      return PK_TYPE_UNKNOWN;
     }
 }
 
+pk_val
+pk_type_name (pk_val type)
+{
+  switch (PVM_VAL_TYP_CODE (type))
+    {
+    case PVM_TYPE_STRUCT:
+      return PVM_VAL_TYP_S_NAME (type);
+      break;
+    default:
+      break;
+    }
+
+  return PK_NULL;
+}
+
+int
+pk_val_kind (pk_val val)
+{
+  if (PVM_IS_INT (val) || PVM_IS_LONG (val))
+    return PK_VAL_INT;
+  else if (PVM_IS_UINT (val) || PVM_IS_ULONG (val))
+    return PK_VAL_UINT;
+  else if (PVM_IS_OFF (val))
+    return PK_VAL_OFFSET;
+  else if (PVM_IS_STR (val))
+    return PK_VAL_STRING;
+  else if (PVM_IS_ARR (val))
+    return PK_VAL_ARRAY;
+  else if (PVM_IS_SCT (val))
+    return PK_VAL_STRUCT;
+  else if (PVM_IS_CLS (val))
+    return PK_VAL_CLOSURE;
+  else if (PVM_IS_TYP (val))
+    return PK_VAL_TYPE;
+  else
+    return PK_VAL_UNKNOWN;
+}
+
 int
 pk_val_equal_p (pk_val val1, pk_val val2)
 {
@@ -272,7 +286,21 @@ pk_struct_nfields (pk_val sct)
   return PVM_VAL_SCT_NFIELDS (sct);
 }
 
-pk_val pk_struct_field_boffset (pk_val sct, uint64_t idx)
+pk_val
+pk_struct_ref_field_value (pk_val sct, const char *fname)
+{
+  return pvm_ref_struct_cstr (sct, fname);
+}
+
+void
+pk_struct_ref_set_field_value (pk_val sct, const char *fname,
+                               pk_val value)
+{
+  pvm_ref_set_struct_cstr (sct, fname, value);
+}
+
+pk_val
+pk_struct_field_boffset (pk_val sct, uint64_t idx)
 {
   if (idx < pk_uint_value (pk_struct_nfields (sct)))
     return PVM_VAL_SCT_FIELD_OFFSET (sct, idx);
@@ -280,13 +308,15 @@ pk_val pk_struct_field_boffset (pk_val sct, uint64_t idx)
     return PK_NULL;
 }
 
-void pk_struct_set_field_boffset (pk_val sct, uint64_t idx, pk_val boffset)
+void
+pk_struct_set_field_boffset (pk_val sct, uint64_t idx, pk_val boffset)
 {
   if (idx < pk_uint_value (pk_struct_nfields (sct)))
     PVM_VAL_SCT_FIELD_OFFSET (sct, idx) = boffset;
 }
 
-pk_val pk_struct_field_name (pk_val sct, uint64_t idx)
+pk_val
+pk_struct_field_name (pk_val sct, uint64_t idx)
 {
   if (idx < pk_uint_value (pk_struct_nfields (sct)))
     return PVM_VAL_SCT_FIELD_NAME (sct, idx);
@@ -294,13 +324,15 @@ pk_val pk_struct_field_name (pk_val sct, uint64_t idx)
     return PK_NULL;
 }
 
-void pk_struct_set_field_name (pk_val sct, uint64_t idx, pk_val name)
+void
+pk_struct_set_field_name (pk_val sct, uint64_t idx, pk_val name)
 {
   if (idx < pk_uint_value (pk_struct_nfields (sct)))
     PVM_VAL_SCT_FIELD_NAME (sct, idx) = name;
 }
 
-pk_val pk_struct_field_value (pk_val sct, uint64_t idx)
+pk_val
+pk_struct_field_value (pk_val sct, uint64_t idx)
 {
   if (idx < pk_uint_value (pk_struct_nfields (sct)))
     return PVM_VAL_SCT_FIELD_VALUE (sct, idx);
@@ -308,7 +340,8 @@ pk_val pk_struct_field_value (pk_val sct, uint64_t idx)
     return PK_NULL;
 }
 
-void pk_struct_set_field_value (pk_val sct, uint64_t idx, pk_val value)
+void
+pk_struct_set_field_value (pk_val sct, uint64_t idx, pk_val value)
 {
   if (idx < pk_uint_value (pk_struct_nfields (sct)))
     PVM_VAL_SCT_FIELD_VALUE (sct, idx) = value;
@@ -362,12 +395,6 @@ pk_offset_type_unit (pk_val type)
   return PVM_VAL_TYP_O_UNIT (type);
 }
 
-pk_val
-pk_make_any_type (void)
-{
-  return pvm_make_any_type ();
-}
-
 pk_val
 pk_make_struct_type (pk_val nfields, pk_val name, pk_val *fnames, pk_val *ftypes)
 {
@@ -461,7 +488,7 @@ pk_array_nelem (pk_val array)
 }
 
 pk_val
-pk_array_elem_val (pk_val array, uint64_t idx)
+pk_array_elem_value (pk_val array, uint64_t idx)
 {
   if (idx < pk_uint_value (pk_array_nelem (array)))
     return PVM_VAL_ARR_ELEM_VALUE (array, idx);
@@ -490,9 +517,8 @@ pk_array_elem_boffset (pk_val array, uint64_t idx)
     return PK_NULL;
 }
 
-void
-pk_array_set_elem_boffset (pk_val array, uint64_t idx, pk_val boffset)
+uint64_t
+pk_sizeof (pk_val val)
 {
-  if (idx < pk_uint_value (pk_array_nelem (array)))
-    PVM_VAL_ARR_ELEM_OFFSET (array, idx) = boffset;
+  return pvm_sizeof (val);
 }
