@@ -1087,6 +1087,20 @@ pkl_ast_node pkl_type_integral_promote (pkl_ast ast,
                                         pkl_ast_node type1,
                                         pkl_ast_node type2);
 
+/* Return the PKL_AST_STRUCT_TYPE_FIELD with the given FIELD_NAME in
+   the given STRUCT_TYPE.  If no such a field exists in the struct
+   type, return NULL.  */
+
+pkl_ast_node pkl_ast_get_struct_type_field (pkl_ast_node struct_type,
+                                            const char *field_name);
+
+/* Return the PKL_AST_DECL node node corresponding to the method named
+   METHOD_NAME in the given STRUCT_TYPE.  If no such method exists
+   then return NULL.  */
+
+pkl_ast_node pkl_ast_get_struct_type_method (pkl_ast_node struct_type,
+                                             const char *method_name);
+
 /* Return an expression that evaluates to an increment step for the
    given TYPE.  If the provided type doesn't support the notion of
    increment step this function returns NULL.  */
@@ -1103,10 +1117,6 @@ pkl_ast_node pkl_ast_type_incr_step (pkl_ast ast, pkl_ast_node type);
    NAME is PKL_AST_IDENTIFIER node containing the name in the
    association.
 
-   PREV_NAME is used in order to cache the name in the declaration
-   when the later is set to "" in re-definition of non-immutable
-   global objects.  See pkl-env.c to see how this field is used.
-
    INITIAL is the initial value of the entity.  The kind of node
    depends on what is being declared:
    - An expression node for a variable.
@@ -1114,6 +1124,13 @@ pkl_ast_node pkl_ast_type_incr_step (pkl_ast ast, pkl_ast_node type);
    - A PKL_AST_FUNC for a function.
    - A constant expression node for an unit.
 
+   PREV_DECL is used to keep track of previous declaration with the
+   same name.  See pkl-env.c to see how this field is used.
+
+   REDECL_CHAIN is used to form a transient sibling relationships among
+   re-declared non-immutable global entities in a compilation session.
+   See pkl-env to see how this field is used.
+
    ORDER is the order of the declaration in its containing
    compile-time environment.  It is filled up when the declaration is
    registered in an environment.
@@ -1130,13 +1147,14 @@ pkl_ast_node pkl_ast_type_incr_step (pkl_ast ast, pkl_ast_node type);
 
 #define PKL_AST_DECL_KIND(AST) ((AST)->decl.kind)
 #define PKL_AST_DECL_NAME(AST) ((AST)->decl.name)
-#define PKL_AST_DECL_PREV_NAME(AST) ((AST)->decl.prev_name)
 #define PKL_AST_DECL_INITIAL(AST) ((AST)->decl.initial)
+#define PKL_AST_DECL_PREV_DECL(AST) ((AST)->decl.prev_decl)
 #define PKL_AST_DECL_ORDER(AST) ((AST)->decl.order)
 #define PKL_AST_DECL_SOURCE(AST) ((AST)->decl.source)
 #define PKL_AST_DECL_STRUCT_FIELD_P(AST) ((AST)->decl.struct_field_p)
 #define PKL_AST_DECL_IN_STRUCT_P(AST) ((AST)->decl.in_struct_p)
 #define PKL_AST_DECL_IMMUTABLE_P(AST) ((AST)->decl.immutable_p)
+#define PKL_AST_DECL_REDECL_CHAIN(AST) ((AST)->decl.redecl_chain)
 
 #define PKL_AST_DECL_KIND_ANY 0
 #define PKL_AST_DECL_KIND_VAR 1
@@ -1153,9 +1171,10 @@ struct pkl_ast_decl
   int in_struct_p;
   int immutable_p;
   char *source;
-  char *prev_name;
   union pkl_ast_node *name;
   union pkl_ast_node *initial;
+  union pkl_ast_node *prev_decl;
+  union pkl_ast_node *redecl_chain;
   int order;
 };
 
