@@ -45,6 +45,8 @@
 #include "pkl-fold.h"
 #include "pkl-env.h"
 
+#include "configmake.h"
+
 #define PKL_COMPILING_EXPRESSION 0
 #define PKL_COMPILING_PROGRAM    1
 #define PKL_COMPILING_STATEMENT  2
@@ -96,17 +98,22 @@ pkl_load_rt (pkl_compiler compiler, char *poke_rt_pk)
 }
 
 pkl_compiler
-pkl_new (pvm vm, const char *rt_path, uint32_t flags)
+pkl_new (pvm vm, const char *rt_path,
+         const char *config_path, uint32_t flags)
 {
   pkl_compiler compiler
     = calloc (1, sizeof (struct pkl_compiler));
+  pkl_env env;
 
   if (!compiler)
     goto out_of_memory;
 
   /* Create the top-level compile-time environment.  This will be used
      for as long as the incremental compiler lives.  */
-  compiler->env = pkl_env_new ();
+   env = pkl_env_new ();
+   if (!env)
+     goto out_of_memory;
+   compiler->env = env;
 
   /* Set the poke virtual machine that the compiler will be generating
      code for.  */
@@ -132,11 +139,11 @@ pkl_new (pvm vm, const char *rt_path, uint32_t flags)
     compiler->bootstrapped = 1;
   }
 
-#define LOAD_STD(NAME)                                          \
+#define LOAD_STD(PATH,NAME)                                     \
   do                                                            \
     {                                                           \
       pvm_val exit_exception;                                   \
-      char *path = pk_str_concat (rt_path, "/" NAME, NULL);     \
+      char *path = pk_str_concat ((PATH), "/" NAME, NULL);      \
       if (!path)                                                \
         goto out_of_memory;                                     \
                                                                 \
@@ -152,11 +159,14 @@ pkl_new (pvm vm, const char *rt_path, uint32_t flags)
     }                                                           \
   while (0)
 
+  /* Load the build configuration of libpoke.  */
+  LOAD_STD (config_path, "pkl-config.pk");
+
   /* Load the standard library.  Note that the standard types may not
      be loaded, depending on how the compiler is configured.  */
-  LOAD_STD ("std.pk");
+  LOAD_STD (rt_path, "std.pk");
   if (!(flags & PKL_F_NOSTDTYPES))
-    LOAD_STD ("std-types.pk");
+    LOAD_STD (rt_path, "std-types.pk");
 
   return compiler;
 
@@ -317,8 +327,10 @@ pkl_execute_buffer (pkl_compiler compiler,
   pkl_env env = NULL;
   pvm_val val;
 
-  compiler->compiling = PKL_COMPILING_PROGRAM;
   env = pkl_env_dup_toplevel (compiler->env);
+  if (!env)
+    goto error;
+  compiler->compiling = PKL_COMPILING_PROGRAM;
 
   /* Parse the input routine into an AST.  */
   ret = pkl_parse_buffer (compiler, &env, &ast,
@@ -348,11 +360,13 @@ pkl_execute_buffer (pkl_compiler compiler,
     {
       pkl_env_free (compiler->env);
       compiler->env = env;
+      pkl_env_commit_renames (compiler->env);
     }
+  else
+    pkl_env_rollback_renames (env);
   return 1;
 
  error:
-  pkl_env_rollback_renames (compiler->env);
   pkl_env_free (env);
   return 0;
 }
@@ -371,8 +385,10 @@ pkl_execute_statement (pkl_compiler compiler,
   int ret;
   pkl_env env = NULL;
 
-  compiler->compiling = PKL_COMPILING_STATEMENT;
   env = pkl_env_dup_toplevel (compiler->env);
+  if (!env)
+    goto error;
+  compiler->compiling = PKL_COMPILING_STATEMENT;
 
   /* Parse the input routine into an AST.  */
   ret = pkl_parse_buffer (compiler, &env, &ast,
@@ -403,11 +419,13 @@ pkl_execute_statement (pkl_compiler compiler,
     {
       pkl_env_free (compiler->env);
       compiler->env = env;
+      pkl_env_commit_renames (compiler->env);
     }
+  else
+    pkl_env_rollback_renames (env);
   return 1;
 
  error:
-  pkl_env_rollback_renames (compiler->env);
   pkl_env_free (env);
   return 0;
 }
@@ -421,8 +439,10 @@ pkl_compile_expression (pkl_compiler compiler,
   int ret;
   pkl_env env = NULL;
 
-   compiler->compiling = PKL_COMPILING_EXPRESSION;
    env = pkl_env_dup_toplevel (compiler->env);
+   if (!env)
+     goto error;
+   compiler->compiling = PKL_COMPILING_EXPRESSION;
 
    /* Parse the input program into an AST.  */
    ret = pkl_parse_buffer (compiler, &env, &ast,
@@ -445,12 +465,12 @@ pkl_compile_expression (pkl_compiler compiler,
 
    pkl_env_free (compiler->env);
    compiler->env = env;
+   pkl_env_commit_renames (compiler->env);
    pvm_program_make_executable (program);
 
    return program;
 
  error:
-   pkl_env_rollback_renames (compiler->env);
    pkl_env_free (env);
    return NULL;
 }
@@ -469,8 +489,10 @@ pkl_execute_expression (pkl_compiler compiler,
   int ret;
   pkl_env env = NULL;
 
-  compiler->compiling = PKL_COMPILING_EXPRESSION;
   env = pkl_env_dup_toplevel (compiler->env);
+  if (!env)
+    goto error;
+  compiler->compiling = PKL_COMPILING_EXPRESSION;
 
   /* Parse the input routine into an AST.  */
   ret = pkl_parse_buffer (compiler, &env, &ast,
@@ -501,11 +523,13 @@ pkl_execute_expression (pkl_compiler compiler,
     {
       pkl_env_free (compiler->env);
       compiler->env = env;
+      pkl_env_commit_renames (compiler->env);
     }
+  else
+    pkl_env_rollback_renames (env);
   return 1;
 
  error:
-  pkl_env_rollback_renames (compiler->env);
   pkl_env_free (env);
   return 0;
 }
@@ -531,6 +555,8 @@ pkl_execute_file (pkl_compiler compiler, const char *fname,
     }
 
   env = pkl_env_dup_toplevel (compiler->env);
+  if (!env)
+    goto error;
   ret = pkl_parse_file (compiler, &env,  &ast, fp, fname);
   if (ret == 1)
     /* Parse error.  */
@@ -554,12 +580,14 @@ pkl_execute_file (pkl_compiler compiler, const char *fname,
     {
       pkl_env_free (compiler->env);
       compiler->env = env;
+      pkl_env_commit_renames (compiler->env);
     }
+  else
+    pkl_env_rollback_renames (env);
   return 1;
 
  error:
   fclose (fp);
-  pkl_env_rollback_renames (compiler->env);
   pkl_env_free (env);
   return 0;
 }
@@ -826,11 +854,11 @@ pvm_type_to_ast_type (pkl_ast ast, pvm_val type)
       break;
     case PVM_TYPE_STRUCT:
       /* XXX writeme */
-      assert (0);
+      PK_UNREACHABLE ();
       break;
     case PVM_TYPE_CLOSURE:
       /* XXX writeme */
-      assert (0);
+      PK_UNREACHABLE ();
       break;
     default:
       break;
@@ -915,3 +943,20 @@ pkl_tracer_p (pkl_compiler compiler)
 
   return PVM_VAL_INT (val);
 }
+
+pkl_ast_node
+pkl_constant_fold (pkl_compiler compiler, pkl_ast ast, pkl_ast_node node)
+{
+  pkl_ast tmp_ast;
+  struct pkl_fold_payload fold_payload = { 0 };
+  struct pkl_phase *fold_phases[] = { &pkl_phase_fold, NULL };
+  void *fold_payloads[] = { &fold_payload, NULL };
+
+  tmp_ast = pkl_ast_init ();
+  tmp_ast->ast = ASTREF (node);
+  tmp_ast->uid = ast->uid;
+  if (!pkl_do_pass (compiler, tmp_ast, fold_phases, fold_payloads, 0, 1))
+    PK_UNREACHABLE ();
+
+  return tmp_ast->ast;
+}
