@@ -62,7 +62,7 @@ extern int pk_map_tab_debug;
     PK_MAP_TAB_EOF = 0,            /* "end of file"  */
     PK_MAP_TAB_error = 256,        /* error  */
     PK_MAP_TAB_UNDEF = 257,        /* "invalid token"  */
-    ERROR = 258,                   /* ERROR  */
+    ERR = 258,                     /* ERR  */
     SEP = 259,                     /* SEP  */
     ENTRY = 260,                   /* ENTRY  */
     TAG = 261,                     /* TAG  */
