@@ -1,6 +1,6 @@
 /* pk-repl.h - A REPL ui for poke.  */
 
-/* Copyright (C) 2019, 2020, 2021 Jose E. Marchesi */
+/* Copyright (C) 2019, 2020, 2021, 2022, 2023 Jose E. Marchesi */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -38,4 +38,14 @@ void pk_repl_display_end (void);
 
 void pk_repl_insert (const char *str);
 
+/* Autocompletion function for Poke artifacts (variables, types,
+   functions, units).  */
+
+char *poke_completion_function (const char *text, int state);
+
+/* Allocate and return an interactive prompt string, e.g.
+   "(poke) " or "#!!# ".
+   The caller is responsible for deallocation with free().  */
+char *pk_prompt(void);
+
 #endif /* ! PK_REPL_H */
