@@ -625,11 +625,11 @@ initialize (int argc, char *argv[])
   /* The Poke hyperlinks facilities must be loaded before poke.pk and
      the cmd subsystem.  This is done even if the hserver is
      disabled.  */
-  if (!pk_load (poke_compiler, "pk-hserver"))
+  if (pk_load (poke_compiler, "pk-hserver") != PK_OK)
     pk_fatal ("unable to load the pk-hserver module");
 
   /* Load poke.pk  */
-  if (!pk_load (poke_compiler, "poke"))
+  if (pk_load (poke_compiler, "poke") != PK_OK)
     pk_fatal ("unable to load the poke module");
 
   /* Set the values of a few global variables defined in poke.pk.  */
