@@ -26,10 +26,12 @@
 #include <err.h>
 #include <getopt.h>
 #include <pthread.h>
+#include <unistd.h>
 
 #include "pk-utils.h"
 #include "usock.h"
 #include "libpoke.h"
+#include "configmake.h"
 
 pk_compiler pkc;
 struct usock *srv;
@@ -479,7 +481,7 @@ poked_version (void);
 static struct poked_options
 {
   int debug_p;
-  const char *socket_path;
+  char *socket_path;
 } poked_options;
 
 static void
@@ -502,7 +504,7 @@ poked_options_init (int argc, char *argv[])
   char c;
   int ret;
 
-  while ((ret = getopt_long (argc, argv, "hvd", options, NULL)) != -1)
+  while ((ret = getopt_long (argc, argv, "hvdS:", options, NULL)) != -1)
     {
       c = ret;
       switch (c)
@@ -523,7 +525,7 @@ poked_options_init (int argc, char *argv[])
           break;
         case OPT_SOCK_PATH:
         case 'S':
-          poked_options.socket_path = optarg;
+          poked_options.socket_path = strdup (optarg);
           break;
         default:
           poked_help ();
@@ -531,7 +533,11 @@ poked_options_init (int argc, char *argv[])
         }
     }
   if (poked_options.socket_path == NULL)
-    poked_options.socket_path = "/tmp/poked.ipc";
+    {
+      if (asprintf (&poked_options.socket_path, "/tmp/poked-%ld.ipc",
+                    (long)getuid ()) == -1)
+        err (1, "asprintf() failed for default socket path");
+    }
 }
 
 static void
@@ -658,6 +664,7 @@ poked_restart:
     }
 
 done:
+  free (poked_options.socket_path);
   poked_free ();
   usock_done (srv);
   pthread_join (th, &ret);
